package org.jcodec.codecs.h264.io.model;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;

import org.jcodec.codecs.h264.io.read.CAVLCReader;
import org.jcodec.codecs.h264.io.write.CAVLCWriter;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * Supplementary Enhanced Information entity of H264 bitstream
 * 
 * capable to serialize and deserialize with CAVLC bitstream
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class SEI extends BitstreamElement {

    public static class SEIMessage {
        public int payloadType;
        public int payloadSize;
        public byte[] payload;

        public SEIMessage(int payloadType2, int payloadSize2, byte[] payload2) {
            this.payload = payload2;
            this.payloadType = payloadType2;
            this.payloadSize = payloadSize2;
        }

    }

    public SEIMessage[] messages;

    public SEI(SEIMessage[] messages) {
        this.messages = messages;
    }

    public static SEI read(InputStream is) throws IOException {
        CAVLCReader reader = new CAVLCReader(is);

        ArrayList messages = new ArrayList();
        do {
            sei_message(reader);
        } while (reader.moreRBSPData());

        reader.readTrailingBits();

        return new SEI((SEIMessage[]) messages.toArray(new SEIMessage[] {}));
    }

    private static SEIMessage sei_message(CAVLCReader reader)
            throws IOException {
        int payloadType = 0;
        while (reader.peakNextBits(8) == 0xFF) {
            reader.readNBit(8);
            payloadType += 255;
        }
        int last_payload_type_byte = (int) reader.readNBit(8,
                "SEI: last_payload_type_byte");
        payloadType += last_payload_type_byte;
        int payloadSize = 0;
        while (reader.peakNextBits(8) == 0xFF) {
            reader.readNBit(8);
            payloadSize += 255;
        }
        int last_payload_size_byte = (int) reader.readNBit(8,
                "SEI: last_payload_size_byte");
        payloadSize += last_payload_size_byte;
        byte[] payload = sei_payload(payloadType, payloadSize, reader);

        return new SEIMessage(payloadType, payloadSize, payload);

    }

    private static byte[] sei_payload(int payloadType, int payloadSize,
            CAVLCReader reader) throws IOException {
        return reader.read(payloadSize);
    }

    public void write(OutputStream out) throws IOException {
        CAVLCWriter writer = new CAVLCWriter(out);
        // TODO Auto-generated method stub

        writer.writeTrailingBits();

    }
}