package org.jcodec.codecs.h264.io.model;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.jcodec.codecs.h264.io.read.CAVLCReader;
import org.jcodec.codecs.h264.io.write.CAVLCWriter;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * Sequence Parameter Set Extension entity of H264 bitstream
 * 
 * Capable to serialize / deserialize itself with CAVLC bit stream
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class SeqParameterSetExt extends BitstreamElement {

    public int seq_parameter_set_id;
    public int aux_format_idc;
    public int bit_depth_aux_minus8;
    public boolean alpha_incr_flag;
    public boolean additional_extension_flag;
    public int alpha_opaque_value;
    public int alpha_transparent_value;

    public static SeqParameterSetExt read(InputStream is) throws IOException {
        CAVLCReader reader = new CAVLCReader(is);

        SeqParameterSetExt spse = new SeqParameterSetExt();
        spse.seq_parameter_set_id = reader.readUE("SPSE: seq_parameter_set_id");
        spse.aux_format_idc = reader.readUE("SPSE: aux_format_idc");
        if (spse.aux_format_idc != 0) {
            spse.bit_depth_aux_minus8 = reader.readUE("SPSE: bit_depth_aux_minus8");
            spse.alpha_incr_flag = reader.readBool("SPSE: alpha_incr_flag");
            spse.alpha_opaque_value = reader.readU(
                    spse.bit_depth_aux_minus8 + 9, "SPSE: alpha_opaque_value");
            spse.alpha_transparent_value = reader.readU(
                    spse.bit_depth_aux_minus8 + 9,
                    "SPSE: alpha_transparent_value");
        }
        spse.additional_extension_flag = reader.readBool("SPSE: additional_extension_flag");

        reader.readTrailingBits();

        return spse;
    }

    public void write(OutputStream out) throws IOException {
        CAVLCWriter writer = new CAVLCWriter(out);
        writer.writeTrailingBits();
    }
}