package org.jcodec.codecs.h264.io.write;

import static org.jcodec.common.tools.Debug.print;
import static org.jcodec.common.tools.Debug.println;

import java.io.IOException;
import java.io.OutputStream;

import org.jcodec.common.io.BitstreamWriter;


/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * A class responsible for outputting exp-Golumb values into binary stream
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class CAVLCWriter extends BitstreamWriter {

    public CAVLCWriter(OutputStream out) {
        super(out);
    }

    public void writeU(int value, int n, String string) throws IOException {
        print(string + "\t");
        writeNBit(value, n);
        println("\t" + value);
    }

    public void writeUE(int value) throws IOException {
        int bits = 0;
        int cumul = 0;
        for (int i = 0; i < 15; i++) {
            if (value < cumul + (1 << i)) {
                bits = i;
                break;
            }
            cumul += (1 << i);
        }
        writeNBit(0, bits);
        write1Bit(1);
        writeNBit(value - cumul, bits);
    }

    public void writeUE(int value, String string) throws IOException {
        print(string + "\t");
        writeUE(value);
        println("\t" + value);
    }

    public void writeSE(int value, String string) throws IOException {
        print(string + "\t");
        int sign = (value >> 31);
        int mod = (value ^ sign) - sign;
        writeUE((mod << 1) - (sign + 1));
        println("\t" + value);
    }

    public void writeBool(boolean value, String string) throws IOException {
        print(string + "\t");
        write1Bit(value ? 1 : 0);
        println("\t" + value);
    }

    public void writeU(int i, int n) throws IOException {
        writeNBit(i, n);
    }
    
    public void writeNBit(long value, int n, String string) throws IOException {
        print(string + "\t");
        for (int i = 0; i < n; i++) {
            write1Bit((int) (value >> (n - i - 1)) & 0x1);
        }
        println("\t" + value);
    }
    
    public void writeTrailingBits() throws IOException {
        write1Bit(1);
        writeRemainingZero();
        flush();
    }

    public void writeSliceTrailingBits() {
        throw new IllegalStateException("todo");
    }
}