package org.jcodec.codecs.mjpeg;

import java.io.PrintStream;

import org.jcodec.codecs.mjpeg.tools.Asserts;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * @author stan
 *
 */
public class HuffTable {
    private final static Type[] values = new Type[32];

    public static enum Type {
        YDC(0x00), CDC(0x01), YAC(0x10), CAC(0x11);
        private final int value;

        Type(int value) {
            this.value = value;
            values[value] = this;
        }

        public static Type getType(int value) {
            return values[value];
        }
    }

    public final static HuffTable YDC_DEFAULT = new HuffTable(Type.YDC.value,
            new short[64 * 1024], new byte[64 * 1024]);
    public final static HuffTable YAC_DEFAULT = new HuffTable(Type.YAC.value,
            new short[64 * 1024], new byte[64 * 1024]);
    public final static HuffTable CDC_DEFAULT = new HuffTable(Type.CDC.value,
            new short[64 * 1024], new byte[64 * 1024]);
    public final static HuffTable CAC_DEFAULT = new HuffTable(Type.CAC.value,
            new short[64 * 1024], new byte[64 * 1024]);
    static {
        YDC_DEFAULT.init();
        YAC_DEFAULT.init();
        CDC_DEFAULT.init();
        CAC_DEFAULT.init();

        YDC_DEFAULT.set(0, 2, "00");
        YDC_DEFAULT.set(1, 3, "010");
        YDC_DEFAULT.set(2, 3, "011");
        YDC_DEFAULT.set(3, 3, "100");
        YDC_DEFAULT.set(4, 3, "101");
        YDC_DEFAULT.set(5, 3, "110");
        YDC_DEFAULT.set(6, 4, "1110");
        YDC_DEFAULT.set(7, 5, "11110");
        YDC_DEFAULT.set(8, 6, "111110");
        YDC_DEFAULT.set(9, 7, "1111110");
        YDC_DEFAULT.set(10, 8, "11111110");
        YDC_DEFAULT.set(11, 9, "111111110");

        CDC_DEFAULT.set(0, 2, "00");
        CDC_DEFAULT.set(1, 2, "01");
        CDC_DEFAULT.set(2, 2, "10");
        CDC_DEFAULT.set(3, 3, "110");
        CDC_DEFAULT.set(4, 4, "1110");
        CDC_DEFAULT.set(5, 5, "11110");
        CDC_DEFAULT.set(6, 6, "111110");
        CDC_DEFAULT.set(7, 7, "1111110");
        CDC_DEFAULT.set(8, 8, "11111110");
        CDC_DEFAULT.set(9, 9, "111111110");
        CDC_DEFAULT.set(10, 10, "1111111110");
        CDC_DEFAULT.set(11, 11, "11111111110");

        YAC_DEFAULT.set(0x00, 4, "1010");
        YAC_DEFAULT.set(0x01, 2, "00");
        YAC_DEFAULT.set(0x02, 2, "01");
        YAC_DEFAULT.set(0x03, 3, "100");
        YAC_DEFAULT.set(0x04, 4, "1011");
        YAC_DEFAULT.set(0x05, 5, "11010");
        YAC_DEFAULT.set(0x06, 7, "1111000");
        YAC_DEFAULT.set(0x07, 8, "11111000");
        YAC_DEFAULT.set(0x08, 10, "1111110110");
        YAC_DEFAULT.set(0x09, 16, "1111111110000010");
        YAC_DEFAULT.set(0x0A, 16, "1111111110000011");
        YAC_DEFAULT.set(0x11, 4, "1100");
        YAC_DEFAULT.set(0x12, 5, "11011");
        YAC_DEFAULT.set(0x13, 7, "1111001");
        YAC_DEFAULT.set(0x14, 9, "111110110");
        YAC_DEFAULT.set(0x15, 11, "11111110110");
        YAC_DEFAULT.set(0x16, 16, "1111111110000100");
        YAC_DEFAULT.set(0x17, 16, "1111111110000101");
        YAC_DEFAULT.set(0x18, 16, "1111111110000110");
        YAC_DEFAULT.set(0x19, 16, "1111111110000111");
        YAC_DEFAULT.set(0x1A, 16, "1111111110001000");

        YAC_DEFAULT.set(0x21, 5, "11100");
        YAC_DEFAULT.set(0x22, 8, "11111001");
        YAC_DEFAULT.set(0x23, 10, "1111110111");
        YAC_DEFAULT.set(0x24, 12, "111111110100");
        YAC_DEFAULT.set(0x25, 16, "1111111110001001");
        YAC_DEFAULT.set(0x26, 16, "1111111110001010");
        YAC_DEFAULT.set(0x27, 16, "1111111110001011");
        YAC_DEFAULT.set(0x28, 16, "1111111110001100");
        YAC_DEFAULT.set(0x29, 16, "1111111110001101");
        YAC_DEFAULT.set(0x2A, 16, "1111111110001110");

        YAC_DEFAULT.set(0x31, 6, "111010");
        YAC_DEFAULT.set(0x32, 9, "111110111");
        YAC_DEFAULT.set(0x33, 12, "111111110101");
        YAC_DEFAULT.set(0x34, 16, "1111111110001111");
        YAC_DEFAULT.set(0x35, 16, "1111111110010000");
        YAC_DEFAULT.set(0x36, 16, "1111111110010001");
        YAC_DEFAULT.set(0x37, 16, "1111111110010010");
        YAC_DEFAULT.set(0x38, 16, "1111111110010011");
        YAC_DEFAULT.set(0x39, 16, "1111111110010100");
        YAC_DEFAULT.set(0x3A, 16, "1111111110010101");

        YAC_DEFAULT.set(0x41, 6, "111011");
        YAC_DEFAULT.set(0x42, 10, "1111111000");
        YAC_DEFAULT.set(0x43, 16, "1111111110010110");
        YAC_DEFAULT.set(0x44, 16, "1111111110010111");
        YAC_DEFAULT.set(0x45, 16, "1111111110011000");
        YAC_DEFAULT.set(0x46, 16, "1111111110011001");
        YAC_DEFAULT.set(0x47, 16, "1111111110011010");
        YAC_DEFAULT.set(0x48, 16, "1111111110011011");
        YAC_DEFAULT.set(0x49, 16, "1111111110011100");
        YAC_DEFAULT.set(0x4A, 16, "1111111110011101");

        YAC_DEFAULT.set(0x51, 7, "1111010");
        YAC_DEFAULT.set(0x52, 11, "11111110111");
        YAC_DEFAULT.set(0x53, 16, "1111111110011110");
        YAC_DEFAULT.set(0x54, 16, "1111111110011111");
        YAC_DEFAULT.set(0x55, 16, "1111111110100000");
        YAC_DEFAULT.set(0x56, 16, "1111111110100001");
        YAC_DEFAULT.set(0x57, 16, "1111111110100010");
        YAC_DEFAULT.set(0x58, 16, "1111111110100011");
        YAC_DEFAULT.set(0x59, 16, "1111111110100100");
        YAC_DEFAULT.set(0x5A, 16, "1111111110100101");

        YAC_DEFAULT.set(0x61, 7, "1111011");
        YAC_DEFAULT.set(0x62, 12, "111111110110");
        YAC_DEFAULT.set(0x63, 16, "1111111110100110");
        YAC_DEFAULT.set(0x64, 16, "1111111110100111");
        YAC_DEFAULT.set(0x65, 16, "1111111110101000");
        YAC_DEFAULT.set(0x66, 16, "1111111110101001");
        YAC_DEFAULT.set(0x67, 16, "1111111110101010");
        YAC_DEFAULT.set(0x68, 16, "1111111110101011");
        YAC_DEFAULT.set(0x69, 16, "1111111110101100");
        YAC_DEFAULT.set(0x6A, 16, "1111111110101101");

        YAC_DEFAULT.set(0x71, 8, "11111010");
        YAC_DEFAULT.set(0x72, 12, "111111110111");
        YAC_DEFAULT.set(0x73, 16, "1111111110101110");
        YAC_DEFAULT.set(0x74, 16, "1111111110101111");
        YAC_DEFAULT.set(0x75, 16, "1111111110110000");
        YAC_DEFAULT.set(0x76, 16, "1111111110110001");
        YAC_DEFAULT.set(0x77, 16, "1111111110110010");
        YAC_DEFAULT.set(0x78, 16, "1111111110110011");
        YAC_DEFAULT.set(0x79, 16, "1111111110110100");
        YAC_DEFAULT.set(0x7A, 16, "1111111110110101");

        YAC_DEFAULT.set(0x81, 9, "111111000");
        YAC_DEFAULT.set(0x82, 15, "111111111000000");
        YAC_DEFAULT.set(0x83, 16, "1111111110110110");
        YAC_DEFAULT.set(0x84, 16, "1111111110110111");
        YAC_DEFAULT.set(0x85, 16, "1111111110111000");
        YAC_DEFAULT.set(0x86, 16, "1111111110111001");
        YAC_DEFAULT.set(0x87, 16, "1111111110111010");
        YAC_DEFAULT.set(0x88, 16, "1111111110111011");
        YAC_DEFAULT.set(0x89, 16, "1111111110111100");
        YAC_DEFAULT.set(0x8A, 16, "1111111110111101");

        YAC_DEFAULT.set(0x91, 9, "111111001");
        YAC_DEFAULT.set(0x92, 16, "1111111110111110");
        YAC_DEFAULT.set(0x93, 16, "1111111110111111");
        YAC_DEFAULT.set(0x94, 16, "1111111111000000");
        YAC_DEFAULT.set(0x95, 16, "1111111111000001");
        YAC_DEFAULT.set(0x96, 16, "1111111111000010");
        YAC_DEFAULT.set(0x97, 16, "1111111111000011");
        YAC_DEFAULT.set(0x98, 16, "1111111111000100");
        YAC_DEFAULT.set(0x99, 16, "1111111111000101");
        YAC_DEFAULT.set(0x9A, 16, "1111111111000110");

        YAC_DEFAULT.set(0xA1, 9, "111111010");
        YAC_DEFAULT.set(0xA2, 16, "1111111111000111");
        YAC_DEFAULT.set(0xA3, 16, "1111111111001000");
        YAC_DEFAULT.set(0xA4, 16, "1111111111001001");
        YAC_DEFAULT.set(0xA5, 16, "1111111111001010");
        YAC_DEFAULT.set(0xA6, 16, "1111111111001011");
        YAC_DEFAULT.set(0xA7, 16, "1111111111001100");
        YAC_DEFAULT.set(0xA8, 16, "1111111111001101");
        YAC_DEFAULT.set(0xA9, 16, "1111111111001110");
        YAC_DEFAULT.set(0xAA, 16, "1111111111001111");

        YAC_DEFAULT.set(0xB1, 10, "1111111001");
        YAC_DEFAULT.set(0xB2, 16, "1111111111010000");
        YAC_DEFAULT.set(0xB3, 16, "1111111111010001");
        YAC_DEFAULT.set(0xB4, 16, "1111111111010010");
        YAC_DEFAULT.set(0xB5, 16, "1111111111010011");
        YAC_DEFAULT.set(0xB6, 16, "1111111111010100");
        YAC_DEFAULT.set(0xB7, 16, "1111111111010101");
        YAC_DEFAULT.set(0xB8, 16, "1111111111010110");
        YAC_DEFAULT.set(0xB9, 16, "1111111111010111");
        YAC_DEFAULT.set(0xBA, 16, "1111111111011000");

        YAC_DEFAULT.set(0xC1, 10, "1111111010");
        YAC_DEFAULT.set(0xC2, 16, "1111111111011001");
        YAC_DEFAULT.set(0xC3, 16, "1111111111011010");
        YAC_DEFAULT.set(0xC4, 16, "1111111111011011");
        YAC_DEFAULT.set(0xC5, 16, "1111111111011100");
        YAC_DEFAULT.set(0xC6, 16, "1111111111011101");
        YAC_DEFAULT.set(0xC7, 16, "1111111111011110");
        YAC_DEFAULT.set(0xC8, 16, "1111111111011111");
        YAC_DEFAULT.set(0xC9, 16, "1111111111100000");
        YAC_DEFAULT.set(0xCA, 16, "1111111111100001");

        YAC_DEFAULT.set(0xD1, 11, "11111111000");
        YAC_DEFAULT.set(0xD2, 16, "1111111111100010");
        YAC_DEFAULT.set(0xD3, 16, "1111111111100011");
        YAC_DEFAULT.set(0xD4, 16, "1111111111100100");
        YAC_DEFAULT.set(0xD5, 16, "1111111111100101");
        YAC_DEFAULT.set(0xD6, 16, "1111111111100110");
        YAC_DEFAULT.set(0xD7, 16, "1111111111100111");
        YAC_DEFAULT.set(0xD8, 16, "1111111111101000");
        YAC_DEFAULT.set(0xD9, 16, "1111111111101001");
        YAC_DEFAULT.set(0xDA, 16, "1111111111101010");

        YAC_DEFAULT.set(0xE1, 16, "1111111111101011");
        YAC_DEFAULT.set(0xE2, 16, "1111111111101100");
        YAC_DEFAULT.set(0xE3, 16, "1111111111101101");
        YAC_DEFAULT.set(0xE4, 16, "1111111111101110");
        YAC_DEFAULT.set(0xE5, 16, "1111111111101111");
        YAC_DEFAULT.set(0xE6, 16, "1111111111110000");
        YAC_DEFAULT.set(0xE7, 16, "1111111111110001");
        YAC_DEFAULT.set(0xE8, 16, "1111111111110010");
        YAC_DEFAULT.set(0xE9, 16, "1111111111110011");
        YAC_DEFAULT.set(0xEA, 16, "1111111111110100");

        YAC_DEFAULT.set(0xF0, 11, "11111111001");
        YAC_DEFAULT.set(0xF1, 16, "1111111111110101");
        YAC_DEFAULT.set(0xF2, 16, "1111111111110110");
        YAC_DEFAULT.set(0xF3, 16, "1111111111110111");
        YAC_DEFAULT.set(0xF4, 16, "1111111111111000");
        YAC_DEFAULT.set(0xF5, 16, "1111111111111001");
        YAC_DEFAULT.set(0xF6, 16, "1111111111111010");
        YAC_DEFAULT.set(0xF7, 16, "1111111111111011");
        YAC_DEFAULT.set(0xF8, 16, "1111111111111100");
        YAC_DEFAULT.set(0xF9, 16, "1111111111111101");
        YAC_DEFAULT.set(0xFA, 16, "1111111111111110");

        CAC_DEFAULT.set(0x00, 2, "00");
        CAC_DEFAULT.set(0x01, 2, "01");
        CAC_DEFAULT.set(0x02, 3, "100");
        CAC_DEFAULT.set(0x03, 4, "1010");
        CAC_DEFAULT.set(0x04, 5, "11000");
        CAC_DEFAULT.set(0x05, 5, "11001");
        CAC_DEFAULT.set(0x06, 6, "111000");
        CAC_DEFAULT.set(0x07, 7, "1111000");
        CAC_DEFAULT.set(0x08, 9, "111110100");
        CAC_DEFAULT.set(0x09, 10, "1111110110");
        CAC_DEFAULT.set(0x0A, 12, "111111110100");

        CAC_DEFAULT.set(0x11, 4, "1011");
        CAC_DEFAULT.set(0x12, 6, "111001");
        CAC_DEFAULT.set(0x13, 8, "11110110");
        CAC_DEFAULT.set(0x14, 9, "111110101");
        CAC_DEFAULT.set(0x15, 11, "11111110110");
        CAC_DEFAULT.set(0x16, 12, "111111110101");
        CAC_DEFAULT.set(0x17, 16, "1111111110001000");
        CAC_DEFAULT.set(0x18, 16, "1111111110001001");
        CAC_DEFAULT.set(0x19, 16, "1111111110001010");
        CAC_DEFAULT.set(0x1A, 16, "1111111110001011");

        CAC_DEFAULT.set(0x21, 5, "11010");
        CAC_DEFAULT.set(0x22, 8, "11110111");
        CAC_DEFAULT.set(0x23, 10, "1111110111");
        CAC_DEFAULT.set(0x24, 12, "111111110110");
        CAC_DEFAULT.set(0x25, 15, "111111111000010");
        CAC_DEFAULT.set(0x26, 16, "1111111110001100");
        CAC_DEFAULT.set(0x27, 16, "1111111110001101");
        CAC_DEFAULT.set(0x28, 16, "1111111110001110");
        CAC_DEFAULT.set(0x29, 16, "1111111110001111");
        CAC_DEFAULT.set(0x2A, 16, "1111111110010000");

        CAC_DEFAULT.set(0x31, 5, "11011");
        CAC_DEFAULT.set(0x32, 8, "11111000");
        CAC_DEFAULT.set(0x33, 10, "1111111000");
        CAC_DEFAULT.set(0x34, 12, "111111110111");
        CAC_DEFAULT.set(0x35, 16, "1111111110010001");
        CAC_DEFAULT.set(0x36, 16, "1111111110010010");
        CAC_DEFAULT.set(0x37, 16, "1111111110010011");
        CAC_DEFAULT.set(0x38, 16, "1111111110010100");
        CAC_DEFAULT.set(0x39, 16, "1111111110010101");
        CAC_DEFAULT.set(0x3A, 16, "1111111110010110");

        CAC_DEFAULT.set(0x41, 6, "111010");
        CAC_DEFAULT.set(0x42, 9, "111110110");
        CAC_DEFAULT.set(0x43, 16, "1111111110010111");
        CAC_DEFAULT.set(0x44, 16, "1111111110011000");
        CAC_DEFAULT.set(0x45, 16, "1111111110011001");
        CAC_DEFAULT.set(0x46, 16, "1111111110011010");
        CAC_DEFAULT.set(0x47, 16, "1111111110011011");
        CAC_DEFAULT.set(0x48, 16, "1111111110011100");
        CAC_DEFAULT.set(0x49, 16, "1111111110011101");
        CAC_DEFAULT.set(0x4A, 16, "1111111110011110");

        CAC_DEFAULT.set(0x51, 6, "111011");
        CAC_DEFAULT.set(0x52, 10, "1111111001");
        CAC_DEFAULT.set(0x53, 16, "1111111110011111");
        CAC_DEFAULT.set(0x54, 16, "1111111110100000");
        CAC_DEFAULT.set(0x55, 16, "1111111110100001");
        CAC_DEFAULT.set(0x56, 16, "1111111110100010");
        CAC_DEFAULT.set(0x57, 16, "1111111110100011");
        CAC_DEFAULT.set(0x58, 16, "1111111110100100");
        CAC_DEFAULT.set(0x59, 16, "1111111110100101");
        CAC_DEFAULT.set(0x5A, 16, "1111111110100110");

        CAC_DEFAULT.set(0x61, 7, "1111001");
        CAC_DEFAULT.set(0x62, 11, "11111110111");
        CAC_DEFAULT.set(0x63, 16, "1111111110100111");
        CAC_DEFAULT.set(0x64, 16, "1111111110101000");
        CAC_DEFAULT.set(0x65, 16, "1111111110101001");
        CAC_DEFAULT.set(0x66, 16, "1111111110101010");
        CAC_DEFAULT.set(0x67, 16, "1111111110101011");
        CAC_DEFAULT.set(0x68, 16, "1111111110101100");
        CAC_DEFAULT.set(0x69, 16, "1111111110101101");
        CAC_DEFAULT.set(0x6A, 16, "1111111110101110");

        CAC_DEFAULT.set(0x71, 7, "1111010");
        CAC_DEFAULT.set(0x72, 11, "11111111000");
        CAC_DEFAULT.set(0x73, 16, "1111111110101111");
        CAC_DEFAULT.set(0x74, 16, "1111111110110000");
        CAC_DEFAULT.set(0x75, 16, "1111111110110001");
        CAC_DEFAULT.set(0x76, 16, "1111111110110010");
        CAC_DEFAULT.set(0x77, 16, "1111111110110011");
        CAC_DEFAULT.set(0x78, 16, "1111111110110100");
        CAC_DEFAULT.set(0x79, 16, "1111111110110101");
        CAC_DEFAULT.set(0x7A, 16, "1111111110110110");

        CAC_DEFAULT.set(0x81, 8, "11111001");
        CAC_DEFAULT.set(0x82, 16, "1111111110110111");
        CAC_DEFAULT.set(0x83, 16, "1111111110111000");
        CAC_DEFAULT.set(0x84, 16, "1111111110111001");
        CAC_DEFAULT.set(0x85, 16, "1111111110111010");
        CAC_DEFAULT.set(0x86, 16, "1111111110111011");
        CAC_DEFAULT.set(0x87, 16, "1111111110111100");
        CAC_DEFAULT.set(0x88, 16, "1111111110111101");
        CAC_DEFAULT.set(0x89, 16, "1111111110111110");
        CAC_DEFAULT.set(0x8A, 16, "1111111110111111");

        CAC_DEFAULT.set(0x91, 9, "111110111");
        CAC_DEFAULT.set(0x92, 16, "1111111111000000");
        CAC_DEFAULT.set(0x93, 16, "1111111111000001");
        CAC_DEFAULT.set(0x94, 16, "1111111111000010");
        CAC_DEFAULT.set(0x95, 16, "1111111111000011");
        CAC_DEFAULT.set(0x96, 16, "1111111111000100");
        CAC_DEFAULT.set(0x97, 16, "1111111111000101");
        CAC_DEFAULT.set(0x98, 16, "1111111111000110");
        CAC_DEFAULT.set(0x99, 16, "1111111111000111");
        CAC_DEFAULT.set(0x9A, 16, "1111111111001000");

        CAC_DEFAULT.set(0xA1, 9, "111111000");
        CAC_DEFAULT.set(0xA2, 16, "1111111111001001");
        CAC_DEFAULT.set(0xA3, 16, "1111111111001010");
        CAC_DEFAULT.set(0xA4, 16, "1111111111001011");
        CAC_DEFAULT.set(0xA5, 16, "1111111111001100");
        CAC_DEFAULT.set(0xA6, 16, "1111111111001101");
        CAC_DEFAULT.set(0xA7, 16, "1111111111001110");
        CAC_DEFAULT.set(0xA8, 16, "1111111111001111");
        CAC_DEFAULT.set(0xA9, 16, "1111111111010000");
        CAC_DEFAULT.set(0xAA, 16, "1111111111010001");

        CAC_DEFAULT.set(0xB1, 9, "111111001");
        CAC_DEFAULT.set(0xB2, 16, "1111111111010010");
        CAC_DEFAULT.set(0xB3, 16, "1111111111010011");
        CAC_DEFAULT.set(0xB4, 16, "1111111111010100");
        CAC_DEFAULT.set(0xB5, 16, "1111111111010101");
        CAC_DEFAULT.set(0xB6, 16, "1111111111010110");
        CAC_DEFAULT.set(0xB7, 16, "1111111111010111");
        CAC_DEFAULT.set(0xB8, 16, "1111111111011000");
        CAC_DEFAULT.set(0xB9, 16, "1111111111011001");
        CAC_DEFAULT.set(0xBA, 16, "1111111111011010");

        CAC_DEFAULT.set(0xC1, 9, "111111010");
        CAC_DEFAULT.set(0xC2, 16, "1111111111011011");
        CAC_DEFAULT.set(0xC3, 16, "1111111111011100");
        CAC_DEFAULT.set(0xC4, 16, "1111111111011101");
        CAC_DEFAULT.set(0xC5, 16, "1111111111011110");
        CAC_DEFAULT.set(0xC6, 16, "1111111111011111");
        CAC_DEFAULT.set(0xC7, 16, "1111111111100000");
        CAC_DEFAULT.set(0xC8, 16, "1111111111100001");
        CAC_DEFAULT.set(0xC9, 16, "1111111111100010");
        CAC_DEFAULT.set(0xCA, 16, "1111111111100011");
        CAC_DEFAULT.set(0xD1, 11, "11111111001");
        CAC_DEFAULT.set(0xD2, 16, "1111111111100100");
        CAC_DEFAULT.set(0xD3, 16, "1111111111100101");
        CAC_DEFAULT.set(0xD4, 16, "1111111111100110");
        CAC_DEFAULT.set(0xD5, 16, "1111111111100111");
        CAC_DEFAULT.set(0xD6, 16, "1111111111101000");
        CAC_DEFAULT.set(0xD7, 16, "1111111111101001");
        CAC_DEFAULT.set(0xD8, 16, "1111111111101010");
        CAC_DEFAULT.set(0xD9, 16, "1111111111101011");
        CAC_DEFAULT.set(0xDA, 16, "1111111111101100");
        CAC_DEFAULT.set(0xE1, 14, "11111111100000");
        CAC_DEFAULT.set(0xE2, 16, "1111111111101101");
        CAC_DEFAULT.set(0xE3, 16, "1111111111101110");
        CAC_DEFAULT.set(0xE4, 16, "1111111111101111");
        CAC_DEFAULT.set(0xE5, 16, "1111111111110000");
        CAC_DEFAULT.set(0xE6, 16, "1111111111110001");
        CAC_DEFAULT.set(0xE7, 16, "1111111111110010");
        CAC_DEFAULT.set(0xE8, 16, "1111111111110011");
        CAC_DEFAULT.set(0xE9, 16, "1111111111110100");
        CAC_DEFAULT.set(0xEA, 16, "1111111111110101");
        CAC_DEFAULT.set(0xF0, 10, "1111111010");
        CAC_DEFAULT.set(0xF1, 15, "111111111000011");
        CAC_DEFAULT.set(0xF2, 16, "1111111111110110");
        CAC_DEFAULT.set(0xF3, 16, "1111111111110111");
        CAC_DEFAULT.set(0xF4, 16, "1111111111111000");
        CAC_DEFAULT.set(0xF5, 16, "1111111111111001");
        CAC_DEFAULT.set(0xF6, 16, "1111111111111010");
        CAC_DEFAULT.set(0xF7, 16, "1111111111111011");
        CAC_DEFAULT.set(0xF8, 16, "1111111111111100");
        CAC_DEFAULT.set(0xF9, 16, "1111111111111101");
        CAC_DEFAULT.set(0xFA, 16, "1111111111111110");

    }
    Type type;
    short[] table;
    byte[] lens;

    public HuffTable(int type, short[] table, byte[] lens) {
        this.table = table;
        this.lens = lens;
        this.type = Type.getType(type);
    }

    private void set(int category, int length, String codeWord) {
        Asserts.assertEquals(length, codeWord.length());
        int i = Integer.parseInt(codeWord, 2);
        table[i] = (short) category;
        lens[i] = (byte) length;
    }

    private final void init() {
        for (int i = 0; i < table.length; i++) {
            table[i] = -1;
        }
    }

    public void debugPrint(PrintStream out) {
        for (int i = 0; i < table.length; i++) {
            int v = table[i];
            int len = lens[i];
            if (v != -1)
                out.println(Integer.toBinaryString(i) + " "
                        + Integer.toHexString(v) + " " + len);
        }
    }
}
