package org.jcodec.common.model;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * A YUV picture
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class Picture {
    private int width;
    private int height;
    private int[] y;
    private int[] cb;
    private int[] cr;

    public Picture(int width, int height, int[] y, int[] cb, int[] cr) {
        this.width = width;
        this.height = height;
        this.y = y;
        this.cb = cb;
        this.cr = cr;
    }

    public static Picture create422(Size size) {
        int pixCnt = size.getWidth() * size.getHeight();
        
        return new Picture(size.getWidth(), size.getHeight(), new int[pixCnt], new int[pixCnt >> 1],
                new int[pixCnt >> 1]);
    }
    
    public static Picture create420(Size size) {
        int pixCnt = size.getWidth() * size.getHeight();
        
        return new Picture(size.getWidth(), size.getHeight(), new int[pixCnt], new int[pixCnt >> 2],
                new int[pixCnt >> 2]);
    }

    public Picture(Picture other) {
        this(other.width, other.height, other.y, other.cb, other.cr);
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    public int[] getY() {
        return y;
    }

    public int[] getCb() {
        return cb;
    }

    public int[] getCr() {
        return cr;
    }

    public Size getYSize() {
        return new Size(width, height);
    }

    public Size getCbSize() {
        return new Size(width >> 1, height);
    }

    public Size getCrSize() {
        return new Size(width >> 1, height);
    }
}
