package org.jcodec.containers.mp4;

import java.io.DataOutput;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jcodec.containers.mp4.boxes.Box;
import org.jcodec.containers.mp4.boxes.BoxFactory;
import org.jcodec.containers.mp4.boxes.Header;
import org.jcodec.containers.mp4.boxes.MovieBox;
import org.jcodec.containers.mp4.boxes.NodeBox;
import org.jcodec.containers.mp4.boxes.TrakBox;
import org.jcodec.containers.mp4.io.FileInput;
import org.jcodec.containers.mp4.io.RandomAccessInput;
import org.jcodec.containers.mp4.io.WindowInput;

public class MP4Util {

    public static MovieBox createRefMovie(RandomAccessInput input, String url) throws IOException {
        MovieBox movie = parseMovie(input);

        for (TrakBox trakBox : movie.getTracks()) {
            trakBox.setDataRef(url);
        }
        return movie;
    }

    public static MovieBox parseMovie(RandomAccessInput input) throws IOException {
        List<Atom> rootAtoms = getRootAtoms(input);
        for (Atom atom : rootAtoms) {
            if ("moov".equals(atom.getHeader().getFourcc())) {
                return (MovieBox) atom.parseBox(input);
            }
        }
        return null;
    }

    public static List<Atom> getRootAtoms(RandomAccessInput input) throws IOException {
        input.seek(0);
        List<Atom> result = new ArrayList<Atom>();
        long off = 0;
        Header atom;
        do {
            atom = Header.read(input);
            if (atom == null)
                break;
            result.add(new Atom(atom, off));
            off += atom.getSize();
            input.seek(off);
        } while (true);

        return result;
    }

    public static class Atom {
        private long offset;
        private Header header;

        public Atom(Header header, long offset) {
            this.header = header;
            this.offset = offset;
        }

        public long getOffset() {
            return offset;
        }

        public Header getHeader() {
            return header;
        }

        public Box parseBox(RandomAccessInput input) throws IOException {
            input.seek(offset + header.headerSize());
            return NodeBox.parseBox(input, header, BoxFactory.getDefault());
        }

        public void copy(RandomAccessInput input, DataOutput out) throws IOException {
            input.seek(offset);
            WindowInput wnd = new WindowInput(input, header.getSize());
            byte[] buf = new byte[8096];
            int read;
            while ((read = wnd.read(buf)) != -1)
                out.write(buf, 0, read);
        }
    }

    public static MovieBox parseMovie(File source) throws IOException {
        FileInput input = null;
        try {
            input = new FileInput(source);
            return parseMovie(input);
        } finally {
            if (input != null)
                input.close();
        }
    }

    public static MovieBox createRefMovie(File source) throws IOException {
        FileInput input = null;
        try {
            input = new FileInput(source);
            return createRefMovie(input, "file://" + source.getCanonicalPath());
        } finally {
            if (input != null)
                input.close();
        }
    }
}