package org.jcodec.containers.mp4.boxes;

import java.io.DataOutput;
import java.io.IOException;

import org.jcodec.containers.mp4.io.Input;
import org.jcodec.containers.mp4.io.Parser;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * A media header atom
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class MediaHeaderBox extends FullBox {
    private long created;
    private long modified;
    private int timescale;
    private long duration;
    private int language;
    private int quality;
    
    public static String fourcc() {
        return "mdhd";
    }

    public MediaHeaderBox(int timescale, long duration, int language, long created, long modified, int quality) {
        super(new Header(fourcc()));
        this.timescale = timescale;
        this.duration = duration;
        this.language = language;
        this.created = created;
        this.modified = modified;
        this.quality = quality;
    }

    public MediaHeaderBox() {
        super(new Header(fourcc()));
    }

    public void parse(Input input) throws IOException {
        super.parse(input);
        if (version == 0) {
            created = Parser.readInt32(input);
            modified = Parser.readInt32(input);
            timescale = (int)Parser.readInt32(input);
            duration = Parser.readInt32(input);
        } else if (version == 1) {
            created = Parser.readInt64(input);
            modified = Parser.readInt64(input);
            timescale = (int)Parser.readInt32(input);
            duration = Parser.readInt64(input);
        } else {
            throw new RuntimeException("Unsupported version");
        }
    }

    public int getTimescale() {
        return timescale;
    }

    public long getDuration() {
        return duration;
    }

    public void doWrite(DataOutput out) throws IOException {
        super.doWrite(out);
        out.writeInt((int) created);
        out.writeInt((int) modified);
        out.writeInt((int) timescale);
        out.writeInt((int) duration);
        out.writeShort((short) language);
        out.writeShort((short) quality);
    }

    public void setDuration(long duration) {
        this.duration = duration;
    }
}