package org.jcodec.containers.mp4.boxes;

import java.io.DataOutput;
import java.io.IOException;

import org.jcodec.containers.mp4.io.Input;
import org.jcodec.containers.mp4.io.Parser;

/**
 * This class is part of JCodec ( www.jcodec.org ) This software is distributed
 * under FreeBSD License
 * 
 * Creates MP4 file out of a set of samples
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class SampleEntry extends NodeBox {

    private short drefInd;

    public SampleEntry(Header header) {
        super(header);
    }

    public SampleEntry(Header header, short drefInd) {
        super(header);
        this.drefInd = drefInd;
    }

    public void parse(Input input) throws IOException {
        input.skip(6); // Reserved
        drefInd = (short) Parser.readInt16(input);
    }
    
    protected void parseExtensions(Input input) throws IOException {
        super.parse(input);
    }

    protected void doWrite(DataOutput out) throws IOException {
        out.write(new byte[] { 0, 0, 0, 0, 0, 0 });
        out.writeShort(drefInd); // data ref index
    }
    
    protected void writeExtensions(DataOutput out) throws IOException {
        super.doWrite(out);
    }

    public short getDrefInd() {
        return drefInd;
    }

    public void setDrefInd(short ind) {
        this.drefInd = ind;
    }
}
