package org.jcodec.containers.mp4.boxes;

import java.io.DataOutput;
import java.io.IOException;

import org.jcodec.containers.mp4.io.Input;
import org.jcodec.containers.mp4.io.Parser;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * Sample to chunk mapping box
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class SampleToChunkBox extends FullBox {

    public static class SampleToChunkEntry {
        private final long first;
        private final int count;
        private int entry;

        public SampleToChunkEntry(long first, int count, int entry) {
            this.first = first;
            this.count = count;
            this.entry = entry;
        }

        public long getFirst() {
            return first;
        }

        public int getCount() {
            return count;
        }

        public int getEntry() {
            return entry;
        }

        public void setEntry(int entry) {
            this.entry = entry;
        }
    }
    
    public static String fourcc() {
        return "stsc";
    }

    private SampleToChunkEntry[] sampleToChunk;

    public SampleToChunkBox(SampleToChunkEntry[] sampleToChunk) {
        super(new Header(fourcc()));
        this.sampleToChunk = sampleToChunk;
    }

    public SampleToChunkBox() {
        super(new Header(fourcc()));
    }

    public void parse(Input input) throws IOException {
        super.parse(input);
        int size = (int) Parser.readInt32(input);

        sampleToChunk = new SampleToChunkEntry[size];
        for (int i = 0; i < size; i++) {
            sampleToChunk[i] = new SampleToChunkEntry(Parser.readInt32(input),
                    (int) Parser.readInt32(input), (int) Parser.readInt32(input));
        }
    }

    public SampleToChunkEntry[] getSampleToChunk() {
        return sampleToChunk;
    }

    @Override
    public void doWrite(DataOutput out) throws IOException {
        super.doWrite(out);
        out.writeInt(sampleToChunk.length);

        for (SampleToChunkEntry stc : sampleToChunk) {
            out.writeInt((int) stc.getFirst());
            out.writeInt((int) stc.getCount());
            out.writeInt((int) stc.getEntry());
        }
    }
}