package org.jcodec.containers.mp4.boxes;

import java.io.DataOutput;
import java.io.IOException;

import org.jcodec.containers.mp4.io.Input;
import org.jcodec.containers.mp4.io.Parser;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * A box containing sample presentation time information
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class TimeToSampleBox extends FullBox {

    public static class TimeToSampleEntry {
        int sampleCount;
        int sampleDuration;

        public TimeToSampleEntry(int sampleCount, int sampleDuration) {
            this.sampleCount = sampleCount;
            this.sampleDuration = sampleDuration;
        }

        public int getSampleCount() {
            return sampleCount;
        }

        public int getSampleDuration() {
            return sampleDuration;
        }
    }
    
    public static String fourcc() {
        return "stts";
    }

    private TimeToSampleEntry[] timeToSamples;

    public TimeToSampleBox(TimeToSampleEntry[] timeToSamples) {
        super(new Header(fourcc()));
        this.timeToSamples = timeToSamples;
    }

    public TimeToSampleBox() {
        super(new Header(fourcc()));
    }

    public void parse(Input input) throws IOException {
        super.parse(input);
        int foo = (int) Parser.readInt32(input);
        timeToSamples = new TimeToSampleEntry[foo];
        for (int i = 0; i < foo; i++) {
            timeToSamples[i] = new TimeToSampleEntry((int) Parser.readInt32(input),
                    (int) Parser.readInt32(input));
        }
    }

    public TimeToSampleEntry[] getTimeToSamples() {
        return timeToSamples;
    }

    @Override
    public void doWrite(DataOutput out) throws IOException {
        super.doWrite(out);
        out.writeInt(timeToSamples.length);
        for (TimeToSampleEntry timeToSampleEntry : timeToSamples) {
            out.writeInt((int) timeToSampleEntry.getSampleCount());
            out.writeInt((int) timeToSampleEntry.getSampleDuration());
        }
    }
}