package org.jcodec.containers.mp4.boxes;

import java.io.DataOutput;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.jcodec.containers.mp4.io.Input;
import org.jcodec.containers.mp4.io.Parser;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * Describes timecode payload sample
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class TimecodeSampleEntry extends SampleEntry {

    private static final MyFactory FACTORY = new MyFactory();
    private int flags;
    private int timescale;
    private int frameDuration;
    private byte numFrames;

    public TimecodeSampleEntry(Header header) {
        super(header);
        factory = FACTORY;
    }

    public void parse(Input input) throws IOException {
        super.parse(input);

        input.skip(4);
        flags = (int) Parser.readInt32(input);
        timescale = (int) Parser.readInt32(input);
        frameDuration = (int) Parser.readInt32(input);
        numFrames = (byte) input.read();
        input.skip(1);
    }

    protected void doWrite(DataOutput out) throws IOException {
        super.doWrite(out);
        out.writeInt(0);
        out.writeInt(flags);
        out.writeInt(timescale);
        out.writeInt(frameDuration);
        out.writeByte(numFrames);
        out.write(207);
    }

    public static class MyFactory extends BoxFactory {
        private Map<String, Class<? extends Box>> mappings = new HashMap<String, Class<? extends Box>>();

        public MyFactory() {
        }

        public Class<? extends Box> toClass(String fourcc) {
            return mappings.get(fourcc);
        }
    }

    public int getFlags() {
        return flags;
    }

    public int getTimescale() {
        return timescale;
    }

    public int getFrameDuration() {
        return frameDuration;
    }

    public byte getNumFrames() {
        return numFrames;
    }
}
