package org.jcodec.containers.mp4.io;

import java.io.IOException;

public class ByteArrayInput implements RandomAccessInput {

    private byte[] bytes;
    private int pos;

    public ByteArrayInput(byte[] bytes) {
        this.bytes = bytes;
    }

    public int read() throws IOException {
        return pos < bytes.length ? bytes[pos++] & 0xff : -1;
    }

    public int read(byte[] buf) throws IOException {
        return read(buf, 0, buf.length);
    }

    public int read(byte[] buf, int off, int len) throws IOException {
        int toRead = Math.min(bytes.length - pos, len);
        System.arraycopy(bytes, pos, buf, off, toRead);
        pos += toRead;
        
        return toRead;
    }

    public void skip(int i) throws IOException {
        pos += i;
    }

    public void seek(long where) throws IOException {
        pos = (int)where;
    }

    public long getPos() throws IOException {
        return pos;
    }
}
