package org.jcodec.containers.mp4.io;

import java.io.IOException;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * An input stream that ensures all reads within a given window
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public class WindowInput implements Input {
    private long remaining = 0;
    private Input proxied;

    public WindowInput(Input in, long remaining) {
        this.remaining = remaining;
        this.proxied = in;
    }

    public void openWindow(int size) {
        remaining = size;
    }

    public int read() throws IOException {
        if (remaining > 0) {
            --remaining;
            return proxied.read();
        } else {
            return -1;
        }
    }

    public void skipRemaining() throws IOException {
        proxied.skip((int) remaining);
    }

    public int read(byte[] buf) throws IOException {
        if(remaining == 0)
            return -1;
        
        long toRead = remaining > buf.length ? buf.length : remaining;
        int res = proxied.read(buf, 0, (int) toRead);
        
        if (res != -1)
            remaining -= res;
        
        return res;
    }

    public int read(byte[] buf, int off, int len) throws IOException {
        if(remaining == 0)
            return -1;
        
        long toRead = remaining > len ? len : remaining;

        int res = proxied.read(buf, off, (int) toRead);
        if (res != -1)
            remaining -= res;
        return res;
    }

    public void skip(int i) throws IOException {
        long toSkip = remaining > i ? i : remaining;
        proxied.skip((int) toSkip);
        remaining -= toSkip;
    }
}