package org.jcodec.movtool;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import org.jcodec.containers.mp4.MP4Util;
import org.jcodec.containers.mp4.MP4Util.Atom;
import org.jcodec.containers.mp4.boxes.MovieBox;
import org.jcodec.containers.mp4.io.FileInput;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * @author Stanislav Vitvitskiy
 * 
 */
public abstract class InplaceEdit {

    protected abstract void apply(MovieBox mov);

    public void save(File f) throws IOException {
        FileInput fi = null;
        RandomAccessFile out = null;
        try {
            fi = new FileInput(f);
            Atom moov = getMoov(fi);
            if (moov == null)
                return;
            MovieBox movBox = (MovieBox) moov.parseBox(fi);
            apply(movBox);
            out = new RandomAccessFile(f, "rw");
            out.seek(moov.getOffset());
            movBox.write(out);
        } finally {
            fi.close();
            out.close();
        }
    }

    private Atom getMoov(FileInput f) throws IOException {
        for (Atom atom : MP4Util.getRootAtoms(f)) {
            if ("moov".equals(atom.getHeader().getFourcc())) {
                return atom;
            }
        }
        return null;
    }
}
