package org.jcodec.scale;

import java.awt.image.BufferedImage;

import org.jcodec.common.model.Picture;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * @author stan
 *
 */
public class RgbToYuv422 implements Transform<BufferedImage, Picture> {

    public void transform(BufferedImage img, Picture dst) {

        int[] Y = dst.getY();
        int[] U = dst.getCb();
        int[] V = dst.getCr();
        
        int[] retY = new int[1], retU = new int[1], retV = new int[1];

        int off = 0;
        for (int i = 0; i < img.getHeight(); i++) {
            for (int j = 0; j < img.getWidth() >> 1; j++) {
                int offY = off << 1;

                int rgb1 = img.getRGB(j << 1, i);
                RGB888toYUV444((rgb1 >> 16) & 0xff, (rgb1 >> 8) & 0xff, rgb1 & 0xff, retY, retU, retV);
                Y[offY] = retY[0] << 2;
                U[off] = retU[0];
                V[off] = retV[0];
                
                int rgb2 = img.getRGB((j << 1) + 1, i);
                RGB888toYUV444((rgb2 >> 16) & 0xff, (rgb2 >> 8) & 0xff, rgb2 & 0xff, retY, retU, retV);
                Y[offY + 1] = retY[0] << 2;
                U[off] += retU[0];
                V[off] += retV[0];

                U[off] <<= 1;
                V[off] <<= 1;
                ++off;
            }
        }
    }

    public static void RGB888toYUV444(int r, int g, int b, int[] Y, int[] U, int[] V) {
        int y = 66 * r + 129 * g + 25 * b;
        int u = -38 * r - 74 * g + 112 * b;
        int v = 112 * r - 94 * g - 18 * b;
        y = (y + 128) >> 8;
        u = (u + 128) >> 8;
        v = (v + 128) >> 8;

        Y[0] = crop(y + 16);
        U[0] = crop(u + 128);
        V[0] = crop(v + 128);
    }

    private static int crop(int val) {
        return val < 0 ? 0 : (val > 255 ? 255 : val);
    }
}