package org.jcodec.scale;

import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;

import org.jcodec.common.model.Picture;

/**
 * This class is part of JCodec ( www.jcodec.org )
 * This software is distributed under FreeBSD License
 * 
 * @author stan
 *
 */
public class Yuv422pToRgb implements Transform<Picture, BufferedImage> {

    public void transform(Picture src, BufferedImage dst) {
        byte[] data = ((DataBufferByte) dst.getRaster().getDataBuffer()).getData();
        int[] y = src.getY();
        int[] u = src.getCb();
        int[] v = src.getCr();

        int offLuma = 0, offChroma = 0;
        for (int i = 0; i < dst.getHeight(); i++) {
            for (int j = 0; j < dst.getWidth(); j += 2) {
                YUV444toRGB888(y[offLuma] >> 2, u[offChroma] >> 2, v[offChroma] >> 2, data, offLuma * 3);
                YUV444toRGB888(y[offLuma + 1] >> 2, u[offChroma] >> 2, v[offChroma] >> 2, data, (offLuma + 1) * 3);
                offLuma += 2;
                ++offChroma;
            }
        }

    }

    public static void YUV444toRGB888(final int y, final int u, final int v, byte[] data, int off) {
        final int c = y - 16;
        final int d = u - 128;
        final int e = v - 128;

        final int r = (298 * c + 409 * e + 128) >> 8;
        final int g = (298 * c - 100 * d - 208 * e + 128) >> 8;
        final int b = (298 * c + 516 * d + 128) >> 8;
        data[off] = (byte) crop(b);
        data[off + 1] = (byte) crop(g);
        data[off + 2] = (byte) crop(r);
    }

    private static int crop(int val) {
        return val < 0 ? 0 : (val > 255 ? 255 : val);
    }
}
